@@ -21,7 +21,6 @@
 #include <gconf/gconf-client.h>
 #include <gtk/gtk.h>
 #include "press_data.h"
-#include "gdk-card-image.h"
 
 /*
  * Constants
@@ -31,6 +30,12 @@
 		     GDK_BUTTON_PRESS_MASK    |\
 		     GDK_POINTER_MOTION_MASK  |\
 		     GDK_BUTTON_RELEASE_MASK)
+
+/* GConf keys. */
+#define WIDTH_GCONF_KEY "/apps/aisleriot/width"
+#define HEIGHT_GCONF_KEY "/apps/aisleriot/height"
+#define THEME_GCONF_KEY "/apps/aisleriot/card_style"
+
 /*
  * Global variables
  */
@@ -39,10 +44,11 @@ extern GtkWidget*       app;
 extern GtkWidget*       playing_area;
 extern GtkWidget*       option_dialog;
 extern GdkGC*           draw_gc;
+extern GdkGC*           slot_gc;
+extern GdkGC*           bg_gc;
 extern GdkPixmap*       surface;
 extern GdkPixmap*       moving_card_pixmap;
-extern GtkObject*       card_deck;
-extern GdkCardDeckOptions deck_options;
+extern gchar*       card_style;
 
 extern guint            score;
 extern guint            game_time;
@@ -52,23 +58,27 @@ extern guint            n_games;
 extern struct dirent**  game_dents;
 extern gchar*           game_file;
 extern gchar*           game_name;
+extern gboolean         game_in_progress;
 extern gboolean         game_over;
 extern gboolean         game_won;
-extern press_data_type* press_data; 
+extern gboolean         click_to_move;
+extern gchar            *gamesdir;
 
 extern guint            x_spacing;
 extern guint            y_spacing;
-extern guint            x_expanded_offset;
-extern guint            y_expanded_offset;
+extern double            x_expanded_offset;
+extern double            y_expanded_offset;
 
 extern GConfClient * gconf_client;
 
 gchar* game_file_to_name(const gchar* file);
 void new_game(gchar* file, guint *seed);
-gboolean quit_app (GtkMenuItem*);
+void quit_app (GtkMenuItem*);
 void set_score( void );
 void timer_start( void );
 void timer_stop( void );
+void timer_reset (void);
+guint timer_get (void);
 void make_title( void );
 void eval_installed_file(gchar *file);
 
