@@ -39,6 +39,49 @@ extern GnibblesProperties *properties;
 
 extern gint current_level;
 
+typedef struct _key_queue_entry {
+	GnibblesWorm *worm;
+	guint keyval;
+} key_queue_entry;
+
+static GQueue *key_queue[NUMWORMS] = {NULL, NULL, NULL, NULL};
+
+static void gnibbles_worm_queue_keypress (GnibblesWorm *worm, guint keyval)
+{
+	key_queue_entry *entry;
+	int n = worm->number;
+
+	if (key_queue[n] == NULL)
+		key_queue[n] = g_queue_new ();
+
+	/* Ignore duplicates in normal movement mode. This resolves the
+	 * key repeat issue. We ignore this in relative mode because then
+	 * you do want two keys that are the same in quick succession. */
+	if ((!properties->wormprops[worm->number]->relmove) && 
+	    (!g_queue_is_empty (key_queue[n])) &&
+	    (keyval == ((key_queue_entry *) g_queue_peek_tail (key_queue[n]))->keyval))
+		return;
+
+	entry = g_malloc (sizeof(key_queue_entry));
+	entry->worm = worm;
+	entry->keyval = keyval;
+	g_queue_push_tail (key_queue[n], (gpointer) entry);
+}
+
+static void gnibbles_worm_queue_empty (GnibblesWorm *worm)
+{
+	key_queue_entry *entry;
+	int n = worm->number;
+
+	if (!key_queue[n])
+		return;
+
+	while (!g_queue_is_empty (key_queue[n])) {
+		entry = g_queue_pop_head (key_queue[n]);
+		g_free (entry);
+	}
+}
+
 GnibblesWorm *
 gnibbles_worm_new (guint t_number)
 {
@@ -77,6 +120,7 @@ gnibbles_worm_set_start (GnibblesWorm *worm, guint t_xhead, guint t_yhead,
 	worm->length = 1;
 	worm->change = SLENGTH - 1;
 	worm->keypress = 0;
+	gnibbles_worm_queue_empty (worm);
 }
 
 gint
@@ -84,8 +128,10 @@ gnibbles_worm_handle_keypress (GnibblesWorm *worm, guint keyval)
 {
 	gint key_left, key_right, key_up, key_down;
 
-	if (worm->keypress)
+	if (worm->keypress) {
+                gnibbles_worm_queue_keypress (worm, keyval);
 		return FALSE;
+	} 
 
 	key_left = gdk_keyval_from_name (properties->wormprops[worm->number]->left);
 	key_right = gdk_keyval_from_name (properties->wormprops[worm->number]->right);
@@ -99,6 +145,7 @@ gnibbles_worm_handle_keypress (GnibblesWorm *worm, guint keyval)
 			worm->direction = worm->direction + 1;
                 else
                         return FALSE;
+		worm->keypress = 1;
 		if (worm->direction == 0)
 			worm->direction = 4;
 		if (worm->direction == 5)
@@ -129,6 +176,18 @@ gnibbles_worm_handle_keypress (GnibblesWorm *worm, guint keyval)
         return FALSE;
 }
 
+static void gnibbles_worm_dequeue_keypress (GnibblesWorm * worm)
+{
+	key_queue_entry *entry;
+	int n = worm->number;
+
+	entry = (key_queue_entry *) g_queue_pop_head (key_queue[n]);
+
+	gnibbles_worm_handle_keypress (entry->worm, entry->keyval);
+
+	g_free (entry);
+}
+
 static gint
 gnibbles_worm_reverse (gpointer data)
 {
@@ -222,9 +281,9 @@ gnibbles_worm_grok_bonus (GnibblesWorm *worm)
 
 void
 gnibbles_worm_draw_head (GnibblesWorm *worm)
-{
+{	
 	worm->keypress = 0;
-	
+
 	switch (worm->direction) {
 		case WORMUP:
 			worm->xoff[worm->start] = 0;
@@ -294,6 +353,11 @@ gnibbles_worm_draw_head (GnibblesWorm *worm)
 
 	gnibbles_draw_pixmap (properties->wormprops[worm->number]->color,
 			worm->xhead, worm->yhead);
+
+	if (key_queue[worm->number] && 
+	    !g_queue_is_empty (key_queue[worm->number])) {
+		gnibbles_worm_dequeue_keypress (worm);
+	}
 }
 
 gint
