@@ -23,11 +23,11 @@
 #include <config.h>
 #include <gnome.h>
 #include <string.h>
-#include <dirent.h>
 #include <gconf/gconf-client.h>
 #include <games-gconf.h>
 #include <games-clock.h>
 #include <games-frame.h>
+#include <games-files.h>
 
 #include "properties.h"
 #include "gnothello.h"
@@ -42,7 +42,6 @@
 #define KEY_SHOW_GRID "/apps/iagno/show_grid"
 #define KEY_FLIP_FINAL_RESULTS "/apps/iagno/flip_final_results"
 
-static GtkWidget *propbox = NULL;
 
 extern GtkWidget *window;
 extern GtkWidget *time_display;
@@ -69,6 +68,8 @@ gint t_animate_stagger;
 gint t_flip_final;
 gint t_grid;
 
+static GamesFileList * theme_file_list = NULL;
+
 static void apply_changes (void);
 
 /*
@@ -239,9 +240,9 @@ reset_properties (void)
 static void 
 black_computer_level_select (GtkWidget *widget, gpointer data)
 {
-	if (((guint) data != t_black_computer_level) 
+	if ((GPOINTER_TO_INT(data) != t_black_computer_level) 
 	    && (GTK_TOGGLE_BUTTON (widget)->active)) {
-		t_black_computer_level = (guint) data;
+		t_black_computer_level = GPOINTER_TO_INT(data);
 		apply_changes ();
 	}
 }
@@ -249,9 +250,9 @@ black_computer_level_select (GtkWidget *widget, gpointer data)
 static void 
 white_computer_level_select (GtkWidget *widget, gpointer data)
 {
-	if (((guint) data != t_white_computer_level)
+	if ((GPOINTER_TO_INT(data) != t_white_computer_level)
 	    && (GTK_TOGGLE_BUTTON (widget)->active)) {
-		t_white_computer_level = (guint) data;
+		t_white_computer_level = GPOINTER_TO_INT(data);
 		apply_changes ();
 	}
 }
@@ -300,7 +301,7 @@ static void
 animate_select (GtkWidget *widget, gpointer data)
 {
 	if (GTK_TOGGLE_BUTTON (widget)->active) {
-		t_animate = (gint) data;
+		t_animate = GPOINTER_TO_INT(data);
 	}
 	apply_changes ();	
 }
@@ -351,12 +352,12 @@ apply_changes (void)
 	white_computer_level = t_white_computer_level;
 	
 	if (black_computer_id) {
-		gtk_timeout_remove (black_computer_id);
+		g_source_remove (black_computer_id);
 		black_computer_id = 0;
 	}
 	
 	if (white_computer_id) {
-		gtk_timeout_remove (white_computer_id);
+		g_source_remove (white_computer_id);
 		white_computer_id = 0;
 	}
 	
@@ -383,21 +384,21 @@ apply_changes (void)
 	animate = t_animate;
 	
 	if (flip_pixmaps_id) {
-		gtk_timeout_remove (flip_pixmaps_id);
+		g_source_remove (flip_pixmaps_id);
 		flip_pixmaps_id = 0;
 	}
 	
 	switch (animate) {
 		case 0:
-			flip_pixmaps_id = gtk_timeout_add (100, flip_pixmaps,
+			flip_pixmaps_id = g_timeout_add (100, flip_pixmaps,
 							   NULL);
 			break;
 		case 1:
-			flip_pixmaps_id = gtk_timeout_add (PIXMAP_FLIP_DELAY * 8,
-							   flip_pixmaps, NULL);
+			flip_pixmaps_id = g_timeout_add (PIXMAP_FLIP_DELAY * 8,
+							 flip_pixmaps, NULL);
 			break;
-		case 2: flip_pixmaps_id = gtk_timeout_add (PIXMAP_FLIP_DELAY,
-							   flip_pixmaps, NULL);
+		case 2: flip_pixmaps_id = g_timeout_add (PIXMAP_FLIP_DELAY,
+							 flip_pixmaps, NULL);
 			break;
 	}
 	
@@ -415,32 +416,27 @@ apply_changes (void)
 	save_properties ();
 }
 
-static void
-close_cb (GtkWidget *widget, gint arg1, gpointer data)
+static gboolean
+close_cb (GtkWidget *widget)
 {
 	gtk_widget_hide (widget);
 
-/*	if (arg1 == GTK_RESPONSE_REJECT)
-		return;
-
-	apply_changes ();
-	
-	save_properties (); */
-}
-
-static void
-destroy_cb (GtkWidget *widget, gpointer data)
-{
-
+	return TRUE;
 }
 
 void
 set_selection (GtkWidget *widget, gpointer data)
 {
-	if (strcmp ((gchar *)data, tile_set_tmp) != 0) {
+	gchar * filename;
+
+	if (tile_set_tmp)
 		g_free (tile_set_tmp);
-		tile_set_tmp = g_strdup (data);
-	}
+
+	filename = games_file_list_get_nth (theme_file_list,
+					    gtk_combo_box_get_active (GTK_COMBO_BOX (widget)));
+		
+	tile_set_tmp = g_strdup (filename);
+
 	apply_changes ();
 }
 
@@ -450,46 +446,29 @@ free_str (GtkWidget *widget, void *data)
         g_free(data);
 }
 
-void
-fill_menu (GtkWidget *menu)
+static GtkWidget *
+fill_menu (void)
 {
-        struct dirent *e;
-        char *dname = NULL;
-        DIR *dir;
-        int itemno = 0;
+        gchar *dname = NULL;
+
+	/* FIXME: we need to check that both dname is valid and that
+	 * games_file_list_new_images returns something. */
 
 	dname = gnome_program_locate_file (NULL,
-			GNOME_FILE_DOMAIN_APP_PIXMAP,  "iagno", FALSE, NULL);
-        dir = opendir(dname);
+					   GNOME_FILE_DOMAIN_APP_PIXMAP, 
+					   "iagno", FALSE, NULL);
 
-        if(!dir)
-                return;
+	if (theme_file_list)
+		g_object_unref (theme_file_list);
 
-        while((e = readdir(dir)) != NULL) {
-                GtkWidget *item;
-                char *s = g_strdup(e->d_name);
-                if(strstr(e->d_name, ".png") == 0) {
-                        g_free(s);
-                        continue;
-                }
-
-                item = gtk_menu_item_new_with_label(s);
-                gtk_widget_show(item);
-                gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
-
-		if (strcmp(tile_set, s) == 0) {
-			gtk_menu_set_active(GTK_MENU(menu), itemno);
-		}
-
-                g_signal_connect(GTK_OBJECT(item), "activate",
-				(GtkSignalFunc)set_selection, s);
-                g_signal_connect(GTK_OBJECT(item), "destroy",
-				(GtkSignalFunc)free_str, s);
-
-                itemno++;
-        }
-        closedir(dir);
+	theme_file_list = games_file_list_new_images (dname, NULL);
 	g_free (dname);
+
+	games_file_list_transform_basename (theme_file_list);
+
+	return games_file_list_create_widget (theme_file_list, tile_set,
+					      GAMES_FILE_LIST_REMOVE_EXTENSION |
+					      GAMES_FILE_LIST_REPLACE_UNDERSCORES);
 }
 
 void
@@ -503,29 +482,27 @@ show_properties_dialog (void)
 	GtkWidget *button;
 	GtkWidget *frame;
 	GtkWidget *vbox, *vbox2;
+	GtkWidget *propbox = NULL;
 	GtkWidget *option_menu;
-	GtkWidget *menu;
 
-	if (propbox)
-	{
+	if (propbox) {
                 gtk_window_present (GTK_WINDOW (propbox));
                 return;
 	}
 
 	reset_properties ();
-	
-	propbox = gtk_dialog_new_with_buttons (NULL,
+
+	propbox = gtk_dialog_new_with_buttons (_("Iagno Preferences"),
 			GTK_WINDOW (window),
 			0,
 			GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);
-        gtk_window_set_title (GTK_WINDOW(propbox), _("Iagno Preferences"));
         
         gtk_dialog_set_has_separator (GTK_DIALOG (propbox), FALSE);
 	notebook = gtk_notebook_new ();
 	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (propbox)->vbox),
                            notebook);
 
-	label = gtk_label_new (_("Players"));
+	label = gtk_label_new_with_mnemonic (_("_Players"));
 
 	vbox = gtk_vbox_new (FALSE, 0);
 	gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
@@ -542,7 +519,7 @@ show_properties_dialog (void)
 	button = gtk_check_button_new_with_label (_("Use quick moves"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
                                       (computer_speed == COMPUTER_MOVE_DELAY / 2));
-	g_signal_connect (GTK_OBJECT (button), "toggled", GTK_SIGNAL_FUNC
+	g_signal_connect (G_OBJECT (button), "toggled", G_CALLBACK
                           (quick_moves_select), NULL);
 	
 	gtk_box_pack_start (GTK_BOX (vbox2), button, FALSE, FALSE, 0);
@@ -558,8 +535,8 @@ show_properties_dialog (void)
 	if (black_computer_level == 0)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 				TRUE);
-	g_signal_connect (GTK_OBJECT (button), "toggled",
-			GTK_SIGNAL_FUNC (black_computer_level_select),
+	g_signal_connect (G_OBJECT (button), "toggled",
+			G_CALLBACK (black_computer_level_select),
 			(gpointer) 0);
 
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
@@ -568,8 +545,8 @@ show_properties_dialog (void)
 	if (black_computer_level == 1)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 				TRUE);
-	g_signal_connect (GTK_OBJECT (button), "toggled",
-			GTK_SIGNAL_FUNC (black_computer_level_select),
+	g_signal_connect (G_OBJECT (button), "toggled",
+			G_CALLBACK (black_computer_level_select),
 			(gpointer) 1);
 
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
@@ -578,8 +555,8 @@ show_properties_dialog (void)
 	if (black_computer_level == 2)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 				TRUE);
-	g_signal_connect (GTK_OBJECT (button), "toggled",
-			GTK_SIGNAL_FUNC (black_computer_level_select),
+	g_signal_connect (G_OBJECT (button), "toggled",
+			G_CALLBACK (black_computer_level_select),
 			(gpointer) 2);
 
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
@@ -588,8 +565,8 @@ show_properties_dialog (void)
 	if (black_computer_level == 3)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 				TRUE);
-	g_signal_connect (GTK_OBJECT (button), "toggled",
-			GTK_SIGNAL_FUNC (black_computer_level_select),
+	g_signal_connect (G_OBJECT (button), "toggled",
+			G_CALLBACK (black_computer_level_select),
 			(gpointer) 3);
 
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
@@ -605,8 +582,8 @@ show_properties_dialog (void)
 	if (white_computer_level == 0)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 				TRUE);
-	g_signal_connect (GTK_OBJECT (button), "toggled",
-			GTK_SIGNAL_FUNC (white_computer_level_select),
+	g_signal_connect (G_OBJECT (button), "toggled",
+			G_CALLBACK (white_computer_level_select),
 			(gpointer) 0);
 
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
@@ -615,8 +592,8 @@ show_properties_dialog (void)
 	if (white_computer_level == 1)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 				TRUE);
-	g_signal_connect (GTK_OBJECT (button), "toggled",
-                          GTK_SIGNAL_FUNC (white_computer_level_select),
+	g_signal_connect (G_OBJECT (button), "toggled",
+                          G_CALLBACK (white_computer_level_select),
                           (gpointer) 1);
 
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
@@ -625,8 +602,8 @@ show_properties_dialog (void)
 	if (white_computer_level == 2)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 				TRUE);
-	g_signal_connect (GTK_OBJECT (button), "toggled",
-                          GTK_SIGNAL_FUNC (white_computer_level_select),
+	g_signal_connect (G_OBJECT (button), "toggled",
+                          G_CALLBACK (white_computer_level_select),
                           (gpointer) 2);
 
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
@@ -636,8 +613,8 @@ show_properties_dialog (void)
 	if (white_computer_level == 3)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 				TRUE);
-	g_signal_connect (GTK_OBJECT (button), "toggled",
-			GTK_SIGNAL_FUNC (white_computer_level_select),
+	g_signal_connect (G_OBJECT (button), "toggled",
+			G_CALLBACK (white_computer_level_select),
 			(gpointer) 3);
 
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
@@ -645,7 +622,7 @@ show_properties_dialog (void)
 	gtk_container_add (GTK_CONTAINER (frame), vbox);
 
 
-        label = gtk_label_new (_("Appearance"));
+        label = gtk_label_new_with_mnemonic (_("_Appearance"));
 
 	table = gtk_table_new (1, 2, FALSE);
 	gtk_container_set_border_width (GTK_CONTAINER (table), 0);
@@ -658,8 +635,8 @@ show_properties_dialog (void)
 	if (animate == 0)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 				TRUE);
-	g_signal_connect (GTK_OBJECT (button), "toggled",
-			GTK_SIGNAL_FUNC (animate_select), (gpointer) 0);
+	g_signal_connect (G_OBJECT (button), "toggled",
+			G_CALLBACK (animate_select), (gpointer) 0);
 
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
 	button = gtk_radio_button_new_with_label (gtk_radio_button_get_group
@@ -667,8 +644,8 @@ show_properties_dialog (void)
 	if (animate == 1)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 				TRUE);
-	g_signal_connect (GTK_OBJECT (button), "toggled",
-			GTK_SIGNAL_FUNC (animate_select), (gpointer) 1);
+	g_signal_connect (G_OBJECT (button), "toggled",
+			G_CALLBACK (animate_select), (gpointer) 1);
 
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
 	button = gtk_radio_button_new_with_label (gtk_radio_button_get_group
@@ -676,8 +653,8 @@ show_properties_dialog (void)
 	if (animate == 2)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 				TRUE);
-	g_signal_connect (GTK_OBJECT (button), "toggled",
-			GTK_SIGNAL_FUNC (animate_select), (gpointer) 2);
+	g_signal_connect (G_OBJECT (button), "toggled",
+			G_CALLBACK (animate_select), (gpointer) 2);
 
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
 	
@@ -690,24 +667,24 @@ show_properties_dialog (void)
 	button = gtk_check_button_new_with_label (_("Stagger flips"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 			t_animate_stagger);
-	g_signal_connect (GTK_OBJECT (button), "toggled",
-			GTK_SIGNAL_FUNC (animate_stagger_select), NULL);
+	g_signal_connect (G_OBJECT (button), "toggled",
+			G_CALLBACK (animate_stagger_select), NULL);
 	
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
 
 	button = gtk_check_button_new_with_label (_("Show grid"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 			t_grid);
-	g_signal_connect (GTK_OBJECT (button), "toggled",
-			GTK_SIGNAL_FUNC (grid_select), NULL);
+	g_signal_connect (G_OBJECT (button), "toggled",
+			G_CALLBACK (grid_select), NULL);
 	
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
 
 	button = gtk_check_button_new_with_label (_("Flip final results"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
 			t_flip_final);
-	g_signal_connect (GTK_OBJECT (button), "toggled",
-			GTK_SIGNAL_FUNC (flip_final_select), NULL);
+	g_signal_connect (G_OBJECT (button), "toggled",
+			G_CALLBACK (flip_final_select), NULL);
 
 	gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
 	
@@ -717,11 +694,9 @@ show_properties_dialog (void)
 	
 	gtk_box_pack_start (GTK_BOX (hbox), label2, FALSE, FALSE, 0);
 	
-	option_menu = gtk_option_menu_new ();
-	menu = gtk_menu_new ();
-	fill_menu (menu);
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
-	
+	option_menu = fill_menu ();
+	g_signal_connect(G_OBJECT(option_menu), "changed",
+			 G_CALLBACK (set_selection), NULL);
 	gtk_box_pack_start (GTK_BOX (hbox), option_menu, TRUE, TRUE, 0);
 	
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
@@ -731,12 +706,10 @@ show_properties_dialog (void)
 	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), table,
                                   label);
         
-	g_signal_connect (GTK_OBJECT (propbox), "response", GTK_SIGNAL_FUNC
+	g_signal_connect (G_OBJECT (propbox), "response", G_CALLBACK
+			(close_cb), NULL);
+	g_signal_connect (G_OBJECT (propbox), "delete_event", G_CALLBACK
 			(close_cb), NULL);
-	g_signal_connect (GTK_OBJECT (propbox), "destroy", GTK_SIGNAL_FUNC
-			(destroy_cb), NULL);
-	g_signal_connect (GTK_OBJECT (propbox), "close", GTK_SIGNAL_FUNC
-			(destroy_cb), NULL);
 
 	gtk_widget_show_all (propbox);
 }
